from .gaokao_time import gk
from .genshin_time import genshin_version_time
from .get_times import now_time, old_time
from .hot_search import hot
from .message import get_msg
from .one_word import get_one_speak
from .starrail_time import starrail_version_time
from .status import system_status
from .year_time import next_year_time

__all__ = [
    "gk",
    "genshin_version_time",
    "starrail_version_time",
    "now_time",
    "old_time",
    "hot",
    "get_msg",
    "get_one_speak",
    "system_status",
    "next_year_time",
]



from nonebot.plugin import PluginMetadata

from .config import env_config
from .main import *  # noqa

__version__ = "1.3.9"
__plugin_meta__ = PluginMetadata(
    name="nonebot_plugin_rename",
    description="用于更改qq机器人的群名片，内置多种有趣名片",
    usage="查看本仓库readme",
    supported_adapters={"~onebot.v11"},
    type="application",
    config=env_config,
    homepage="https://github.com/forchannot/nonebot_plugin_rename",
    extra={
        "author": "forchannot",
        "version": __version__,
    },
)



import asyncio
import random
from pathlib import Path
from typing import List

from nonebot import get_bots, get_driver, logger, on_command, require
from nonebot.adapters.onebot.v11 import (
    GROUP_ADMIN,
    GROUP_OWNER,
    ActionFailed,
    Bot,
    GroupMessageEvent,
    Message,
    MessageSegment,
    PrivateMessageEvent,
)
from nonebot.drivers import Driver
from nonebot.params import CommandArg
from nonebot.permission import SUPERUSER

from .config import env_config
from .utils import card_list, choice_card, generate_card_image, read_yaml, write_yaml

require("nonebot_plugin_apscheduler")
from nonebot_plugin_apscheduler import scheduler  # noqa

driver: Driver = get_driver()
hour, minute = env_config.set_group_card_hour, env_config.set_group_card_minute
if driver.config.nickname:
    NICKNAME = env_config.self_name or list(driver.config.nickname)[0]
else:
    NICKNAME = env_config.self_name or "bot"
yml_file = Path.cwd() / "data" / "group_card"
permissions = SUPERUSER | GROUP_ADMIN | GROUP_OWNER

group_card = on_command(
    "设置群名片",
    aliases={"更改群名片", "修改群名片"},
    permission=permissions,
    priority=13,
    block=True,
)
view_pic = on_command(
    "查看群名片列表",
    aliases={"查看所有群名片", "群名片列表"},
    permission=permissions,
    priority=14,
    block=True,
)
view_card = on_command(
    "查看当前群名片",
    aliases={"当前群名片"},
    permission=permissions,
    priority=14,
    block=True,
)
set_card_now = on_command(
    "立即更改群名片",
    aliases={"立即设置群名片", "立即修改群名片"},
    permission=permissions,
    priority=10,
    block=True,
)
del_group_card = on_command(
    "删除群名片",
    aliases={"删除本群群名片"},
    permission=permissions,
    priority=10,
    block=True,
)
set_all_group_card = on_command(
    "更改所有群名片",
    aliases={"设置所有群名片", "修改所有群名片"},
    permission=SUPERUSER,
    priority=10,
    block=True,
)


# on_command "设置群名片"
@group_card.handle()
async def get_group_card(bot: Bot, event: GroupMessageEvent):
    # 解析参数
    group_nicknames = str(event.get_message()).strip().split()[1:]
    nicks = " ".join(group_nicknames)
    if not group_nicknames:
        await group_card.finish("请输入你想要设置的群名片序号")
    group_id = str(event.group_id)
    bot_id = bot.self_id
    # 读取群名片数据
    group_data = read_yaml(yml_file / "group_card.yaml") or {}
    group_nicknames_valid = any(
        int(gn) > len(card_list) for gn in group_nicknames
    )  # 判断用户输入的群名片序号是否有效
    # 更新群名片数据
    if group_nicknames_valid:
        await group_card.finish("没有这种群名片哦")
    group_data.setdefault(bot_id, {})
    group_data[bot_id].setdefault(group_id, {})
    group_data[bot_id][group_id] = group_nicknames
    write_yaml(yml_file / "group_card.yaml", group_data)
    await group_card.finish(f"已为你更改该群群名片序号为{nicks}")


# 定时任务执行函数
async def set_group_card(is_handle: bool = False):
    tasks = []
    set_wrong = []
    bots = get_bots()
    group_data = read_yaml(yml_file / "group_card.yaml") or {}
    if not group_data:
        return
    for bot_id, bot_case in bots.items():
        if group_info := group_data.get(bot_id, {}):
            tasks.extend(await set_card(group_info, bot_id, bot_case))
    results = await asyncio.gather(*tasks, return_exceptions=True)
    for group_info, result in zip(group_data.values(), results):  # noqa: B905
        group_id = next(iter(group_info))
        if isinstance(result, Exception):
            logger.warning(f"群{group_id}名片更改失败，错误信息：{result}")
            set_wrong.append(group_id)
    if is_handle:
        return set_wrong


# on_command "查看群名片列表"
@view_pic.handle()
async def _(event: GroupMessageEvent):
    img = MessageSegment.image(generate_card_image())
    await view_pic.finish(message=MessageSegment.text("可以使用<更改群名片 序号>进行设置") + img)


# on_command "查看当前群名片"
@view_card.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    group_data = read_yaml(yml_file / "group_card.yaml") or {}
    img = MessageSegment.image(generate_card_image())
    if str(event.group_id) in group_data.get(bot.self_id, {}):
        result = group_data[bot.self_id][str(event.group_id)]
        nicks = " ".join(result)
        result = f"当前群组设置的群名片序号有{nicks}"
    else:
        result = "当前没有设置群名片哦,请先发送<设置群名片 序号>命令进行设置吧"
    await view_card.finish(message=MessageSegment.text(result) + img)


# on_command "立即更改群名片"
@set_card_now.handle()
async def _(
    bot: Bot, event: GroupMessageEvent, arg: Message = CommandArg()
):  # noqa: B008
    card_number = arg.extract_plain_text().strip()
    if not card_number:
        await set_card_now.finish("请输入序号或序号输入错误")
    elif card_number not in map(str, range(1, len(card_list) + 1)):
        await set_card_now.finish("没有这种类型的群名片哦，可以发送[查看群名片列表]命令查看吧")
    card_names = await choice_card(card_number)
    if env_config.use_nickname_front:
        card_names = f"{NICKNAME}|{card_names}"
    try:
        await bot.set_group_card(
            group_id=event.group_id, user_id=int(bot.self_id), card=card_names
        )
        logger.info(f"群组{event.group_id}成功设置名片 >> {card_names}")
    except (AttributeError, ActionFailed):
        logger.warning("更改群名片失败，可能是机器人不存在或被风控")


# on_command "删除群名片"
@del_group_card.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    group_data = read_yaml(yml_file / "group_card.yaml") or {}
    if bot.self_id in group_data and str(event.group_id) in group_data[bot.self_id]:
        group_data[bot.self_id].pop(str(event.group_id))
        write_yaml(yml_file / "group_card.yaml", group_data)
        await del_group_card.finish("删除成功")
    else:
        await del_group_card.finish("本群还没设置过群名片哦")


# on_command "设置所有群名片"
@set_all_group_card.handle()
async def _(bot: Bot, event: PrivateMessageEvent):
    if env_config.is_one_bot_set_all_group_card:
        result = await set_group_card(is_handle=True)
        msg = f"群名片设置成功,设置失败的群有\n{' '.join(result)}" if result else "所有群名片设置成功"
        await set_all_group_card.finish(msg)
    set_wrong = []
    group_data = read_yaml(yml_file / "group_card.yaml") or {}
    group_info = group_data.get(bot.self_id, {})
    if not group_info:
        await set_all_group_card.finish("没有设置过任何群名片哦")
    tasks = await set_card(group_info, bot.self_id, bot)
    results = await asyncio.gather(*tasks, return_exceptions=True)
    for group_info, result in zip(group_data.values(), results):  # noqa: B905
        group_id = next(iter(group_info))
        if isinstance(result, Exception):
            logger.warning(f"群{group_id}名片更改失败，错误信息：{result}")
            set_wrong.append(group_id)
    msg = f"群名片设置成功,设置失败的群有\n{' '.join(set_wrong)}" if set_wrong else "所有群名片设置成功"
    await set_all_group_card.finish(msg)


async def set_card(group_info: dict, bot_id: str, bot_case) -> List:
    tasks = []
    for group_id, group_nicks in group_info.items():
        card_names = await choice_card(random.choice(group_nicks))
        if env_config.use_nickname_front:
            card_names = f"{NICKNAME}|{card_names}"
        if card_names:
            tasks.append(
                bot_case.set_group_card(
                    group_id=group_id,
                    user_id=int(bot_id),
                    card=card_names,
                )
            )
            if env_config.is_show_aps_info_log:
                logger.info(f"即将为群{group_id}的bot设置群名片后缀{card_names}")
    return tasks


# 定时任务入口
@scheduler.scheduled_job("interval", hours=hour, minutes=minute, id="rename_group_card")
async def _():
    if env_config.is_show_aps_info_log:
        logger.info("开始为列表中的群更改bot群名片")
    await set_group_card()


# bot启动时执行
@driver.on_startup
async def init_group_card():
    if not (yml_file / "group_card.yaml").exists():
        yml_file.mkdir(parents=True, exist_ok=True)
        (yml_file / "group_card.yaml").touch()
        logger.info("创建group_card.yaml文件成功")



# Description: 获取一言

from typing import Optional

import httpx
from nonebot import logger


async def get_one_speak() -> Optional[str]:
    async with httpx.AsyncClient(follow_redirects=True) as client:
        try:
            res = await client.get("https://v1.hitokoto.cn/")
            if res.status_code != 200:
                logger.warning(f"获取一言失败: {res.status_code}")
                return None
            data = res.json()["hitokoto"]
            return data[:16]
        except Exception as e:
            logger.warning(f"获取一言失败: {e}")
            return None



import datetime


def next_year_time() -> str:
    # 获取当前日期和时间
    now = datetime.datetime.now()

    # 指定目标年份
    target_year = now.year + 1  # 下一年

    # 构造目标日期
    target_date = datetime.datetime(target_year, 1, 1)

    # 计算时间差
    time_difference = target_date - now

    # 计算剩余天、小时、分钟和秒
    total_seconds = time_difference.total_seconds()
    days = int(total_seconds // (24 * 3600))
    hours = int((total_seconds % (24 * 3600)) // 3600)
    minutes = int((total_seconds % 3600) // 60)

    # 输出结果
    return f"距离{target_year}年还有{days}天{hours}小时{minutes}分"



# Description: 获取机器人所在系统状态

import asyncio

import psutil


async def system_status():
    mem = psutil.virtual_memory()
    # 系统总计内存
    zj = float(mem.total) / 1024 / 1024 / 1024
    # 系统已经使用内存
    ysy = float(mem.used) / 1024 / 1024 / 1024
    # 系统空闲内存
    # kx = round(float(mem.free) / 1024 / 1024 / 1024, 1)
    # 系统已使用内存百分比
    mem_percent = round(ysy / zj * 100, 1)
    psutil.cpu_percent()
    await asyncio.sleep(0.1)
    cpu_percent = psutil.cpu_percent()
    return f"内存:{mem_percent}%|CPU:{cpu_percent}%"



# Description: 获取机器人收发消息数量

from nonebot import get_bot


async def get_msg() -> str:
    bot = get_bot()
    bot_status = await bot.get_status()
    if bot_status := bot_status.get("stat"):
        msg_received = bot_status.get("message_received", "未知")
        msg_sent = bot_status.get("message_sent", "未知")
        return f"已发送{msg_sent}条|已接收{msg_received}条"
    return "获取bot收发消息失败"



from io import BytesIO
from pathlib import Path

from PIL import Image, ImageDraw, ImageFont

from .card_name import card_list


font_path = Path(__file__).parent.parent / "fonts" / "draw.ttf"


def generate_card_image(
    font: Path = font_path,
    font_size: int = 20,
    title_left: str = "群名片序号",
    title_right: str = "群名片描述",
):
    # 创建一个空白的图片
    img = Image.new("RGB", (800, len(card_list) * 50 + 50), color="white")
    # 设置字体和字号
    font = ImageFont.truetype(font.as_posix(), size=font_size)
    # 创建一个绘制对象
    draw = ImageDraw.Draw(img)
    # 绘制标题和分割线
    draw.text((50, 10), title_left, font=font, fill="black")
    draw.line((200, 0, 200, len(card_list) * 50 + 50), fill="black")
    draw.text((220, 10), title_right, font=font, fill="black")
    draw.line((0, 50, 800, 50), fill="black")
    # 遍历列表，获取内容并绘制到图片上
    for i in range(len(card_list)):
        if card_list[str(i + 1)][0]:
            draw.text((95, i * 50 + 65), str(i + 1), font=font, fill="black")
            draw.line((200, i * 50 + 50, 200, (i + 1) * 50 + 50), fill="black")
            draw.text(
                (220, i * 50 + 65), card_list[str(i + 1)][0], font=font, fill="black"
            )
            draw.line((0, (i + 1) * 50 + 50, 800, (i + 1) * 50 + 50), fill="black")
    # 返回base64编码后的图片字符串
    img_buffer = BytesIO()
    img.save(img_buffer, format="PNG")
    return img_buffer.getvalue()



# Description: 插件的配置文件
from nonebot import get_driver
from pydantic import BaseModel, Extra
from typing import Optional


class Config(BaseModel, extra=Extra.ignore):
    set_group_card_hour: int = 0
    set_group_card_minute: int = 30
    use_nickname_front: bool = True
    self_name: Optional[str] = None
    is_one_bot_set_all_group_card: bool = False
    is_show_aps_info_log: bool = True


env_config = Config.parse_obj(get_driver().config.dict())



# Description: 获取当前时间
import datetime


def old_time() -> str:
    now = datetime.datetime.now()
    BigHourName = ["子", "丑", "寅", "卯", "辰", "巳", "午", "未", "申", "酉", "戌", "亥", "子"]
    BigHourType = ["正", "初"]
    # BigMinName = ["零", "一", "二", "三", "四"]
    hour = now.hour
    # minutes = now.minute
    return f"现在是长安{BigHourName[(hour + 1) // 2]}{BigHourType[hour % 2]}"


def now_time() -> str:
    now = datetime.datetime.now()
    hour = now.hour
    # minutes = now.minute
    if hour < 10:
        hour = f"0{hour}"
    # if minutes < 10:
    #     minutes = f"0{minutes}"
    return f"现在是北京时间{hour}点"



# Description: 获取原神版本剩余时间
from datetime import datetime


def genshin_version_time() -> str:
    # 原神版本号
    Versions = [36, 37, 38, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50]
    Index = 0
    # 获取持续时间
    baseTime = datetime.strptime("2023-4-12 11:00:00", "%Y-%m-%d %H:%M:%S")
    nowTime = datetime.now()
    duringTime = (baseTime - nowTime).total_seconds() * 1000
    # 推算版本
    while duringTime <= 0:
        duringTime += 42 * 24 * 60 * 60 * 1000
        Index += 1
    # 计算版本号并取到小数点后一位
    Version = (Versions[Index] / 10).__round__(1)
    # 获取天数并取整
    days = int(duringTime / (24 * 3600 * 1000))
    leave1 = duringTime % (24 * 3600 * 1000)
    # 获取小时数并取整
    hours = int(leave1 / (3600 * 1000))
    leave2 = leave1 % (3600 * 1000)
    # 获取分钟数并取整
    minutes = int(leave2 / (60 * 1000))
    # 字符串处理
    return f"离原神{Version}还有{days}天{hours}小时{minutes}分钟"



# Description: 获取星穹铁道版本剩余时间
from datetime import datetime


def starrail_version_time() -> str:
    # 基准版本
    Version = 1.1
    baseTime = (
        datetime.strptime("2023-6-7 11:00:00", "%Y-%m-%d %H:%M:%S").timestamp() * 1000
    )
    nowTime = datetime.now().timestamp() * 1000
    # 获取持续时间
    duringTime = baseTime - nowTime
    while duringTime <= 0:
        # 版本+0.1 同时时间+42天
        duringTime += 42 * 24 * 60 * 60 * 1000
        Version += 0.1
    # 获取天数并取整
    days = int(duringTime / (24 * 3600 * 1000))
    leave1 = duringTime % (24 * 3600 * 1000)
    # 获取小时数并取整
    hours = int(leave1 / (3600 * 1000))
    leave2 = leave1 % (3600 * 1000)
    # 获取分钟数并取整
    minutes = int(leave2 / (60 * 1000))
    return f"离崩铁{Version:.1f}还有{days}天{hours}小时{minutes}分钟"



import datetime


def gk() -> str:
    now = datetime.datetime.now()
    current_year = now.year

    # 设置目标年份的高考日期，每年的6月7日9点到6月8日17点
    gk_start = datetime.datetime(current_year, 6, 7, 9)
    gk_end = datetime.datetime(current_year, 6, 8, 17)

    # 判断是否已经超过当年的高考日期
    if now > gk_end:
        current_year += 1
        gk_start = datetime.datetime(current_year, 6, 7, 9)
        gk_end = datetime.datetime(current_year, 6, 8, 17)

    if gk_start <= now <= gk_end:
        return f"{current_year}年高考正在进行中"
    # 计算剩余时间
    remaining_time = gk_start - now
    days = remaining_time.days
    hours, remainder = divmod(remaining_time.seconds, 3600)
    minutes, _ = divmod(remainder, 60)
    return f"距离{current_year}年高考还有{days}天{hours}小时{minutes}分钟"



# Description: 根据卡片数字获取卡片名称
from inspect import iscoroutinefunction

from .card_name import card_list


async def choice_card(num) -> str:
    card_name_choice = card_list.get(num, "ee")[1]
    if card_name_choice == "e":
        return "没有这种类型"
    arg = (int(num) - 3,) if 4 <= int(num) <= 9 else ()
    return (
        await card_name_choice(*arg)
        if iscoroutinefunction(card_name_choice)
        else card_name_choice(*arg)  # type: ignore
    )



# Description: 获取热搜

import random
from typing import Optional

import httpx
from nonebot import logger

URL = {
    "1": "https://tenapi.cn/v2/bilihot/",  # B站
    "2": "https://tenapi.cn/v2/weibohot",  # 微博
    "3": "https://tenapi.cn/v2/douyinhot",  # 抖音
    "4": "https://tenapi.cn/v2/baiduhot/",  # 百度
    "5": "https://tenapi.cn/v2/zhihuhot",  # 知乎
    "6": "https://tenapi.cn/v2/toutiaohot",  # 今日头条
}


async def hot(num: int) -> Optional[str]:
    async with httpx.AsyncClient(follow_redirects=True) as client:
        try:
            res = await client.get(URL[str(num)])
            if res.status_code != 200:
                return "热搜api失效"
            data = res.json()["data"]
            result = random.choice(data)["name"]
            return result[:16]
        except Exception as e:
            logger.warning(f"获取热搜失败: {e}")
            return None



from .card_choice import choice_card
from .card_name import card_list
from .draw import generate_card_image
from .my_yaml import read_yaml, write_yaml

__all__ = [
    "card_list",
    "choice_card",
    "generate_card_image",
    "read_yaml",
    "write_yaml",
]



# Description: 群名片序号对应的函数
from ..card import (
    genshin_version_time,
    get_msg,
    get_one_speak,
    gk,
    hot,
    now_time,
    old_time,
    system_status,
    starrail_version_time,
    next_year_time,
)

card_list = {
    "1": ("原神版本剩余时间", genshin_version_time),
    "2": ("北京时间(小时)", now_time),
    "3": ("当前时间(古代计时制)", old_time),
    "4": ("B站热搜", hot),
    "5": ("微博热搜", hot),
    "6": ("抖音热搜", hot),
    "7": ("百度热搜", hot),
    "8": ("知乎热搜", hot),
    "9": ("今日头条热搜", hot),
    "10": ("每日一言", get_one_speak),
    "11": ("距离高考剩余时间", gk),
    "12": ("bot系统运行状态", system_status),
    "13": ("bot收发消息统计", get_msg),
    "14": ("崩铁版本剩余时间", starrail_version_time),
    "15": ("距离下一年时间", next_year_time),
}



# Description: YAML文件处理

import yaml


# 读取Yaml文件方法
def read_yaml(yaml_path) -> dict:
    with open(yaml_path, encoding="utf-8", mode="r") as f:
        return yaml.load(stream=f, Loader=yaml.FullLoader)


# 写入YAML文件的方法
def write_yaml(yaml_path, data):
    with open(yaml_path, encoding="utf-8", mode="w") as f:
        yaml.dump(data, stream=f, allow_unicode=True)
